@@ -6,7 +6,7 @@ library analysis.server;
 
 import 'dart:async';
 import 'dart:collection';
-import 'dart:core' hide Resource;
+import 'dart:core';
 import 'dart:io' as io;
 import 'dart:math' show max;
 
@@ -26,16 +26,14 @@ import 'package:analysis_server/src/services/search/search_engine_internal.dart'
 import 'package:analysis_server/src/single_context_manager.dart';
 import 'package:analyzer/dart/ast/ast.dart';
 import 'package:analyzer/dart/element/element.dart';
+import 'package:analyzer/exception/exception.dart';
 import 'package:analyzer/file_system/file_system.dart';
 import 'package:analyzer/instrumentation/instrumentation.dart';
 import 'package:analyzer/plugin/resolver_provider.dart';
 import 'package:analyzer/source/pub_package_map_provider.dart';
 import 'package:analyzer/src/context/builder.dart';
 import 'package:analyzer/src/dart/ast/utilities.dart';
-import 'package:analyzer/src/dart/sdk/sdk.dart';
 import 'package:analyzer/src/generated/engine.dart';
-import 'package:analyzer/src/generated/java_engine.dart';
-import 'package:analyzer/src/generated/java_io.dart';
 import 'package:analyzer/src/generated/sdk.dart';
 import 'package:analyzer/src/generated/source.dart';
 import 'package:analyzer/src/generated/source_io.dart';
@@ -46,7 +44,6 @@ import 'package:analyzer/src/task/dart.dart';
 import 'package:analyzer/src/util/glob.dart';
 import 'package:analyzer/task/dart.dart';
 import 'package:plugin/plugin.dart';
-import 'package:yaml/yaml.dart';
 
 typedef void OptionUpdater(AnalysisOptionsImpl options);
 
@@ -83,7 +80,7 @@ class AnalysisServer {
    * The version of the analysis server. The value should be replaced
    * automatically during the build.
    */
-  static final String VERSION = '1.15.0';
+  static final String VERSION = '1.17.0';
 
   /**
    * The number of milliseconds to perform operations before inserting
@@ -300,6 +297,12 @@ class AnalysisServer {
    */
   ResolverProvider fileResolverProvider;
 
+  /**
+   * The package resolver provider used to override the way package URI's are
+   * resolved in some contexts.
+   */
+  ResolverProvider packageResolverProvider;
+
   /**
    * The manager of pub package summaries.
    */
@@ -353,6 +356,7 @@ class AnalysisServer {
           defaultContextOptions);
     }
     this.fileResolverProvider = fileResolverProvider;
+    this.packageResolverProvider = packageResolverProvider;
     ServerContextManagerCallbacks contextManagerCallbacks =
         new ServerContextManagerCallbacks(this, resourceProvider);
     contextManager.callbacks = contextManagerCallbacks;
@@ -372,8 +376,9 @@ class AnalysisServer {
     _setupIndexInvalidation();
     pubSummaryManager =
         new PubSummaryManager(resourceProvider, '${io.pid}.temp');
-    Notification notification =
-        new ServerConnectedParams(VERSION, io.pid).toNotification();
+    Notification notification = new ServerConnectedParams(VERSION, io.pid,
+            sessionId: instrumentationService.sessionId)
+        .toNotification();
     channel.sendNotification(notification);
     channel.listen(handleRequest, onDone: done, onError: error);
     handlers = serverPlugin.createDomains(this);
@@ -396,7 +401,7 @@ class AnalysisServer {
       for (String pattern in patterns) {
         try {
           _analyzedFilesGlobs
-              .add(new Glob(JavaFile.pathContext.separator, pattern));
+              .add(new Glob(resourceProvider.pathContext.separator, pattern));
         } catch (exception, stackTrace) {
           AnalysisEngine.instance.logger.logError(
               'Invalid glob pattern: "$pattern"',
@@ -472,7 +477,7 @@ class AnalysisServer {
    * The socket from which requests are being read has been closed.
    */
   void done() {
-    index.stop();
+    index?.stop();
     running = false;
   }
 
@@ -723,7 +728,7 @@ class AnalysisServer {
       return units;
     }
     // add a unit for each unit/library combination
-    runWithWorkingCacheSize(context, () {
+    runWithActiveContext(context, () {
       Source unitSource = contextSource.source;
       List<Source> librarySources = context.getLibrariesContaining(unitSource);
       for (Source librarySource in librarySources) {
@@ -1463,7 +1468,7 @@ class AnalysisServer {
       return null;
     }
     // if library has been already resolved, resolve unit
-    return runWithWorkingCacheSize(context, () {
+    return runWithActiveContext(context, () {
       return context.resolveCompilationUnit2(source, librarySource);
     });
   }
@@ -1594,17 +1599,11 @@ class ServerContextManagerCallbacks extends ContextManagerCallbacks {
   ServerContextManagerCallbacks(this.analysisServer, this.resourceProvider);
 
   @override
-  AnalysisContext addContext(
-      Folder folder, AnalysisOptions options, FolderDisposition disposition) {
-    InternalAnalysisContext context =
-        AnalysisEngine.instance.createAnalysisContext();
-    context.contentCache = analysisServer.overlayState;
-    analysisServer.folderMap[folder] = context;
-    context.fileResolverProvider = analysisServer.fileResolverProvider;
-    context.sourceFactory =
-        _createSourceFactory(context, options, disposition, folder);
-    context.analysisOptions = options;
+  AnalysisContext addContext(Folder folder, AnalysisOptions options) {
+    ContextBuilder builder = createContextBuilder(folder, options);
+    AnalysisContext context = builder.buildContext(folder.path);
 
+    // TODO(brianwilkerson) Move bundle discovery into ContextBuilder
     if (analysisServer.options.enablePubSummaryManager) {
       List<LinkedPubPackage> linkedBundles =
           analysisServer.pubSummaryManager.getLinkedBundles(context);
@@ -1614,11 +1613,12 @@ class ServerContextManagerCallbacks extends ContextManagerCallbacks {
           store.addBundle(null, package.unlinked);
           store.addBundle(null, package.linked);
         }
-        context.resultProvider =
+        (context as InternalAnalysisContext).resultProvider =
             new InputPackagesResultProvider(context, store);
       }
     }
 
+    analysisServer.folderMap[folder] = context;
     analysisServer._onContextsChangedController
         .add(new ContextsChangedEvent(added: [context]));
     analysisServer.schedulePerformAnalysisOperation(context);
@@ -1630,10 +1630,8 @@ class ServerContextManagerCallbacks extends ContextManagerCallbacks {
   void applyChangesToContext(Folder contextFolder, ChangeSet changeSet) {
     AnalysisContext context = analysisServer.folderMap[contextFolder];
     if (context != null) {
-      ApplyChangesStatus changesStatus = context.applyChanges(changeSet);
-      if (changesStatus.hasChanges) {
-        analysisServer.schedulePerformAnalysisOperation(context);
-      }
+      context.applyChanges(changeSet);
+      analysisServer.schedulePerformAnalysisOperation(context);
       List<String> flushedFiles = new List<String>();
       for (Source source in changeSet.removedSources) {
         flushedFiles.add(source.fullName);
@@ -1646,6 +1644,33 @@ class ServerContextManagerCallbacks extends ContextManagerCallbacks {
   void computingPackageMap(bool computing) =>
       analysisServer._computingPackageMap(computing);
 
+  @override
+  ContextBuilder createContextBuilder(Folder folder, AnalysisOptions options) {
+    String defaultPackageFilePath = null;
+    String defaultPackagesDirectoryPath = null;
+    String path = (analysisServer.contextManager as ContextManagerImpl)
+        .normalizedPackageRoots[folder.path];
+    if (path != null) {
+      Resource resource = resourceProvider.getResource(path);
+      if (resource.exists) {
+        if (resource is File) {
+          defaultPackageFilePath = path;
+        } else {
+          defaultPackagesDirectoryPath = path;
+        }
+      }
+    }
+
+    ContextBuilder builder = new ContextBuilder(resourceProvider,
+        analysisServer.sdkManager, analysisServer.overlayState);
+    builder.defaultOptions = options;
+    builder.fileResolverProvider = analysisServer.fileResolverProvider;
+    builder.packageResolverProvider = analysisServer.packageResolverProvider;
+    builder.defaultPackageFilePath = defaultPackageFilePath;
+    builder.defaultPackagesDirectoryPath = defaultPackagesDirectoryPath;
+    return builder;
+  }
+
   @override
   void moveContext(Folder from, Folder to) {
     // There is nothing to do.
@@ -1672,54 +1697,6 @@ class ServerContextManagerCallbacks extends ContextManagerCallbacks {
         .add(new ContextsChangedEvent(changed: [context]));
     analysisServer.schedulePerformAnalysisOperation(context);
   }
-
-  /**
-   * Set up a [SourceFactory] that resolves packages as appropriate for the
-   * given [disposition].
-   */
-  SourceFactory _createSourceFactory(InternalAnalysisContext context,
-      AnalysisOptions options, FolderDisposition disposition, Folder folder) {
-    List<UriResolver> resolvers = [];
-    List<UriResolver> packageUriResolvers =
-        disposition.createPackageUriResolvers(resourceProvider);
-
-    // If no embedded URI resolver was provided, defer to a locator-backed one.
-    EmbedderYamlLocator locator =
-        disposition.getEmbedderLocator(resourceProvider);
-    Map<Folder, YamlMap> embedderYamls = locator.embedderYamls;
-    EmbedderSdk embedderSdk = new EmbedderSdk(resourceProvider, embedderYamls);
-    if (embedderSdk.libraryMap.size() == 0) {
-      // There was no embedder file, or the file was empty, so used the default
-      // SDK.
-      resolvers.add(new DartUriResolver(
-          analysisServer.sdkManager.getSdkForOptions(options)));
-    } else {
-      // The embedder file defines an alternate SDK, so use it.
-      List<String> paths = <String>[];
-      for (Folder folder in embedderYamls.keys) {
-        paths.add(folder
-            .getChildAssumingFile(EmbedderYamlLocator.EMBEDDER_FILE_NAME)
-            .path);
-      }
-      DartSdk dartSdk = analysisServer.sdkManager
-          .getSdk(new SdkDescription(paths, options), () {
-        embedderSdk.analysisOptions = options;
-        // TODO(brianwilkerson) Enable summary use after we have decided where
-        // summary files for embedder files will live.
-        embedderSdk.useSummary = false;
-        return embedderSdk;
-      });
-      resolvers.add(new DartUriResolver(dartSdk));
-    }
-
-    resolvers.addAll(packageUriResolvers);
-    if (context.fileResolverProvider == null) {
-      resolvers.add(new ResourceUriResolver(resourceProvider));
-    } else {
-      resolvers.add(context.fileResolverProvider(folder));
-    }
-    return new SourceFactory(resolvers, disposition.packages);
-  }
 }
 
 /**
