@@ -7,9 +7,10 @@ library context.directory.manager;
 import 'dart:async';
 import 'dart:collection';
 import 'dart:convert';
-import 'dart:core' hide Resource;
+import 'dart:core';
 
 import 'package:analysis_server/src/analysis_server.dart';
+import 'package:analyzer/exception/exception.dart';
 import 'package:analyzer/file_system/file_system.dart';
 import 'package:analyzer/instrumentation/instrumentation.dart';
 import 'package:analyzer/plugin/options.dart';
@@ -23,10 +24,8 @@ import 'package:analyzer/source/pub_package_map_provider.dart';
 import 'package:analyzer/source/sdk_ext.dart';
 import 'package:analyzer/src/context/builder.dart';
 import 'package:analyzer/src/context/context.dart' as context;
-import 'package:analyzer/src/context/source.dart';
 import 'package:analyzer/src/dart/sdk/sdk.dart';
 import 'package:analyzer/src/generated/engine.dart';
-import 'package:analyzer/src/generated/java_engine.dart';
 import 'package:analyzer/src/generated/java_io.dart';
 import 'package:analyzer/src/generated/sdk.dart';
 import 'package:analyzer/src/generated/source.dart';
@@ -323,11 +322,9 @@ abstract class ContextManager {
 abstract class ContextManagerCallbacks {
   /**
    * Create and return a new analysis context rooted at the given [folder], with
-   * the given analysis [options], allowing [disposition] to govern details of
-   * how the context is to be created.
+   * the given analysis [options].
    */
-  AnalysisContext addContext(
-      Folder folder, AnalysisOptions options, FolderDisposition disposition);
+  AnalysisContext addContext(Folder folder, AnalysisOptions options);
 
   /**
    * Called when the set of files associated with a context have changed (or
@@ -342,6 +339,12 @@ abstract class ContextManagerCallbacks {
    */
   void computingPackageMap(bool computing);
 
+  /**
+   * Create and return a context builder that can be used to create a context
+   * for the files in the given [folder] when analyzed using the given [options].
+   */
+  ContextBuilder createContextBuilder(Folder folder, AnalysisOptions options);
+
   /**
    * Called when the context manager changes the folder with which a context is
    * associated. Currently this is mostly FYI, and used only in tests.
@@ -882,13 +885,11 @@ class ContextManagerImpl implements ContextManager {
     if (AnalysisEngine.isAnalysisOptionsFileName(path, pathContext)) {
       var analysisContext = info.context;
       if (analysisContext is context.AnalysisContextImpl) {
-        // TODO(brianwilkerson) This doesn't correctly update the source factory
-        // if the changes necessitate it (such as by changing the setting of the
-        // strong-mode option).
         Map<String, Object> options = readOptions(info.folder);
         processOptionsForContext(info, options,
             optionsRemoved: changeType == ChangeType.REMOVE);
-        analysisContext.invalidateCachedResults();
+        analysisContext.sourceFactory = _createSourceFactory(
+            analysisContext, analysisContext.analysisOptions, info.folder);
         callbacks.applyChangesToContext(info.folder, new ChangeSet());
       }
     }
@@ -899,55 +900,14 @@ class ContextManagerImpl implements ContextManager {
     // Check to see if this is the .packages file for this context and if so,
     // update the context's source factory.
     if (absolutePathContext.basename(path) == PACKAGE_SPEC_NAME) {
-      File packagespec = resourceProvider.getFile(path);
-      if (packagespec.exists) {
-        // Locate embedder yamls for this .packages file.
-        // If any embedder libs are contributed and this context does not
-        // have an embedded URI resolver, we need to create a new context.
-
-        List<int> bytes = packagespec.readAsStringSync().codeUnits;
-        Map<String, Uri> packages =
-            pkgfile.parse(bytes, new Uri.file(packagespec.path));
-
-        Map<String, List<Folder>> packageMap =
-            new PackagesFileDisposition(new MapPackages(packages))
-                .buildPackageMap(resourceProvider);
-        Map<Folder, YamlMap> embedderYamls =
-            new EmbedderYamlLocator(packageMap).embedderYamls;
-
-        SourceFactory sourceFactory = info.context.sourceFactory;
-
-        // Check for library embedders.
-        if (embedderYamls.values.any(definesEmbeddedLibs)) {
-          // If there is no embedded URI resolver, a new source factory needs to
-          // be recreated.
-          if (sourceFactory is SourceFactoryImpl) {
-            // Get all but the dart: Uri resolver.
-            List<UriResolver> resolvers = sourceFactory.resolvers
-                .where((r) => r is! DartUriResolver)
-                .toList();
-            // Add an embedded URI resolver in its place.
-            resolvers.add(new DartUriResolver(
-                new EmbedderSdk(resourceProvider, embedderYamls)));
-
-            // Set a new source factory.
-            SourceFactoryImpl newFactory = sourceFactory.clone();
-            newFactory.resolvers.clear();
-            newFactory.resolvers.addAll(resolvers);
-            info.context.sourceFactory = newFactory;
-            return;
-          }
-        }
-
-        // Next check for package URI updates.
-        if (info.isPathToPackageDescription(path)) {
-          Packages packages = _readPackagespec(packagespec);
-          if (packages != null) {
-            _updateContextPackageUriResolver(
-                folder, new PackagesFileDisposition(packages));
-          }
-        }
-      }
+      String contextRoot = info.folder.path;
+      ContextBuilder builder =
+          callbacks.createContextBuilder(info.folder, defaultContextOptions);
+      AnalysisOptions options =
+          builder.getAnalysisOptions(info.context, contextRoot);
+      SourceFactory factory = builder.createSourceFactory(contextRoot, options);
+      info.context.analysisOptions = options;
+      info.context.sourceFactory = factory;
     }
   }
 
@@ -1073,7 +1033,7 @@ class ContextManagerImpl implements ContextManager {
     applyToAnalysisOptions(options, optionMap);
 
     info.setDependencies(dependencies);
-    info.context = callbacks.addContext(folder, options, disposition);
+    info.context = callbacks.addContext(folder, options);
     folderMap[folder] = info.context;
     info.context.name = folder.path;
 
@@ -1164,42 +1124,10 @@ class ContextManagerImpl implements ContextManager {
    * Set up a [SourceFactory] that resolves packages as appropriate for the
    * given [disposition].
    */
-  SourceFactory _createSourceFactory(InternalAnalysisContext context,
-      AnalysisOptions options, FolderDisposition disposition, Folder folder) {
-    List<UriResolver> resolvers = [];
-    List<UriResolver> packageUriResolvers =
-        disposition.createPackageUriResolvers(resourceProvider);
-
-    EmbedderYamlLocator locator =
-        disposition.getEmbedderLocator(resourceProvider);
-    Map<Folder, YamlMap> embedderYamls = locator.embedderYamls;
-    EmbedderSdk embedderSdk = new EmbedderSdk(resourceProvider, embedderYamls);
-    if (embedderSdk.libraryMap.size() == 0) {
-      // There was no embedder file, or the file was empty, so used the default
-      // SDK.
-      resolvers.add(new DartUriResolver(sdkManager.getSdkForOptions(options)));
-    } else {
-      // The embedder file defines an alternate SDK, so use it.
-      List<String> paths = <String>[];
-      for (Folder folder in embedderYamls.keys) {
-        paths.add(folder
-            .getChildAssumingFile(EmbedderYamlLocator.EMBEDDER_FILE_NAME)
-            .path);
-      }
-      DartSdk dartSdk =
-          sdkManager.getSdk(new SdkDescription(paths, options), () {
-        embedderSdk.analysisOptions = options;
-        // TODO(brianwilkerson) Enable summary use after we have decided where
-        // summary files for embedder files will live.
-        embedderSdk.useSummary = false;
-        return embedderSdk;
-      });
-      resolvers.add(new DartUriResolver(dartSdk));
-    }
-
-    resolvers.addAll(packageUriResolvers);
-    resolvers.add(new ResourceUriResolver(resourceProvider));
-    return new SourceFactory(resolvers, disposition.packages);
+  SourceFactory _createSourceFactory(
+      InternalAnalysisContext context, AnalysisOptions options, Folder folder) {
+    ContextBuilder builder = callbacks.createContextBuilder(folder, options);
+    return builder.createSourceFactory(folder.path, options);
   }
 
   /**
@@ -1584,10 +1512,8 @@ class ContextManagerImpl implements ContextManager {
     // while we're rerunning "pub list", since any analysis we complete while
     // "pub list" is in progress is just going to get thrown away anyhow.
     List<String> dependencies = <String>[];
-    FolderDisposition disposition = _computeFolderDisposition(
-        info.folder, dependencies.add, _findPackageSpecFile(info.folder));
     info.setDependencies(dependencies);
-    _updateContextPackageUriResolver(info.folder, disposition);
+    _updateContextPackageUriResolver(info.folder);
   }
 
   /**
@@ -1627,11 +1553,10 @@ class ContextManagerImpl implements ContextManager {
     return null;
   }
 
-  void _updateContextPackageUriResolver(
-      Folder contextFolder, FolderDisposition disposition) {
+  void _updateContextPackageUriResolver(Folder contextFolder) {
     AnalysisContext context = folderMap[contextFolder];
-    context.sourceFactory = _createSourceFactory(
-        context, context.analysisOptions, disposition, contextFolder);
+    context.sourceFactory =
+        _createSourceFactory(context, context.analysisOptions, contextFolder);
     callbacks.updateContextPackageUriResolver(context);
   }
 
@@ -1710,6 +1635,10 @@ class CustomPackageResolverDisposition extends FolderDisposition {
   @override
   EmbedderYamlLocator getEmbedderLocator(ResourceProvider resourceProvider) =>
       new EmbedderYamlLocator(null);
+
+  @override
+  SdkExtensionFinder getSdkExtensionFinder(ResourceProvider resourceProvider) =>
+      new SdkExtensionFinder(null);
 }
 
 /**
@@ -1757,6 +1686,13 @@ abstract class FolderDisposition {
    * where that is necessary.
    */
   EmbedderYamlLocator getEmbedderLocator(ResourceProvider resourceProvider);
+
+  /**
+   * Return the extension finder used to locate the `_sdkext` file used to add
+   * extensions to the SDK. The [resourceProvider] is used to access the file
+   * system in cases where that is necessary.
+   */
+  SdkExtensionFinder getSdkExtensionFinder(ResourceProvider resourceProvider);
 }
 
 /**
@@ -1780,6 +1716,10 @@ class NoPackageFolderDisposition extends FolderDisposition {
   @override
   EmbedderYamlLocator getEmbedderLocator(ResourceProvider resourceProvider) =>
       new EmbedderYamlLocator(null);
+
+  @override
+  SdkExtensionFinder getSdkExtensionFinder(ResourceProvider resourceProvider) =>
+      new SdkExtensionFinder(null);
 }
 
 /**
@@ -1790,6 +1730,7 @@ class PackageMapDisposition extends FolderDisposition {
   final Map<String, List<Folder>> packageMap;
 
   EmbedderYamlLocator _embedderLocator;
+  SdkExtensionFinder _sdkExtensionFinder;
 
   @override
   final String packageRoot;
@@ -1814,6 +1755,11 @@ class PackageMapDisposition extends FolderDisposition {
     }
     return _embedderLocator;
   }
+
+  @override
+  SdkExtensionFinder getSdkExtensionFinder(ResourceProvider resourceProvider) {
+    return _sdkExtensionFinder ??= new SdkExtensionFinder(packageMap);
+  }
 }
 
 /**
@@ -1827,6 +1773,7 @@ class PackagesFileDisposition extends FolderDisposition {
   Map<String, List<Folder>> packageMap;
 
   EmbedderYamlLocator _embedderLocator;
+  SdkExtensionFinder _sdkExtensionFinder;
 
   PackagesFileDisposition(this.packages);
 
@@ -1868,4 +1815,10 @@ class PackagesFileDisposition extends FolderDisposition {
     }
     return _embedderLocator;
   }
+
+  @override
+  SdkExtensionFinder getSdkExtensionFinder(ResourceProvider resourceProvider) {
+    return _sdkExtensionFinder ??=
+        new SdkExtensionFinder(buildPackageMap(resourceProvider));
+  }
 }
