@@ -6,7 +6,7 @@ library analysis_server.src.services.correction.fix_internal;
 
 import 'dart:async';
 import 'dart:collection';
-import 'dart:core' hide Resource;
+import 'dart:core';
 
 import 'package:analysis_server/plugin/edit/fix/fix_core.dart';
 import 'package:analysis_server/plugin/edit/fix/fix_dart.dart';
@@ -28,14 +28,15 @@ import 'package:analyzer/dart/ast/ast.dart';
 import 'package:analyzer/dart/ast/token.dart';
 import 'package:analyzer/dart/element/element.dart';
 import 'package:analyzer/dart/element/type.dart';
+import 'package:analyzer/error/error.dart';
 import 'package:analyzer/file_system/file_system.dart';
 import 'package:analyzer/src/dart/ast/token.dart';
 import 'package:analyzer/src/dart/ast/utilities.dart';
 import 'package:analyzer/src/dart/element/element.dart';
 import 'package:analyzer/src/dart/element/member.dart';
 import 'package:analyzer/src/dart/element/type.dart';
+import 'package:analyzer/src/error/codes.dart';
 import 'package:analyzer/src/generated/engine.dart';
-import 'package:analyzer/src/generated/error.dart';
 import 'package:analyzer/src/generated/java_core.dart';
 import 'package:analyzer/src/generated/parser.dart';
 import 'package:analyzer/src/generated/sdk.dart';
@@ -518,9 +519,10 @@ class FixProcessor {
   }
 
   void _addFix_addPartOfDirective() {
+    // TODO(brianwilkerson) Generalize this to allow other valid string literals.
     if (node is SimpleStringLiteral && node.parent is PartDirective) {
       PartDirective directive = node.parent;
-      Source partSource = directive.source;
+      Source partSource = directive.uriSource;
       CompilationUnit partUnit;
       partUnit = context.getResolvedCompilationUnit2(partSource, partSource);
       if (partUnit != null) {
@@ -697,8 +699,8 @@ class FixProcessor {
       }
     }
     // prepare location for a new constructor
-    _ClassMemberLocation targetLocation =
-        _prepareNewConstructorLocation(classDeclaration);
+    ClassMemberLocation targetLocation =
+        utils.prepareNewConstructorLocation(classDeclaration);
     // build constructor source
     SourceBuilder sb = new SourceBuilder(file, targetLocation.offset);
     {
@@ -741,8 +743,8 @@ class FixProcessor {
     if (targetTypeNode is! ClassDeclaration) {
       return;
     }
-    _ClassMemberLocation targetLocation =
-        _prepareNewConstructorLocation(targetTypeNode);
+    ClassMemberLocation targetLocation =
+        utils.prepareNewConstructorLocation(targetTypeNode);
     String targetFile = targetElement.source.fullName;
     // build method source
     SourceBuilder sb = new SourceBuilder(targetFile, targetLocation.offset);
@@ -797,8 +799,8 @@ class FixProcessor {
     if (targetTypeNode is! ClassDeclaration) {
       return;
     }
-    _ClassMemberLocation targetLocation =
-        _prepareNewConstructorLocation(targetTypeNode);
+    ClassMemberLocation targetLocation =
+        utils.prepareNewConstructorLocation(targetTypeNode);
     String targetFile = targetElement.source.fullName;
     // build method source
     SourceBuilder sb = new SourceBuilder(targetFile, targetLocation.offset);
@@ -863,7 +865,7 @@ class FixProcessor {
       }
       // add super constructor name
       sb.append('super');
-      if (!StringUtils.isEmpty(constructorName)) {
+      if (!isEmpty(constructorName)) {
         sb.append('.');
         sb.append(constructorName);
       }
@@ -935,8 +937,8 @@ class FixProcessor {
         argumentsBuffer.append(parameterName);
       }
       // add proposal
-      _ClassMemberLocation targetLocation =
-          _prepareNewConstructorLocation(targetClassNode);
+      ClassMemberLocation targetLocation =
+          utils.prepareNewConstructorLocation(targetClassNode);
       SourceBuilder sb = new SourceBuilder(file, targetLocation.offset);
       {
         sb.append(targetLocation.prefix);
@@ -1017,8 +1019,8 @@ class FixProcessor {
     }
     ClassDeclaration targetClassNode = targetTypeNode;
     // prepare location
-    _ClassMemberLocation targetLocation =
-        _prepareNewFieldLocation(targetClassNode);
+    ClassMemberLocation targetLocation =
+        utils.prepareNewFieldLocation(targetClassNode);
     // build method source
     String targetFile = targetClassElement.source.fullName;
     SourceBuilder sb = new SourceBuilder(targetFile, targetLocation.offset);
@@ -1150,8 +1152,8 @@ class FixProcessor {
     }
     ClassDeclaration targetClassNode = targetTypeNode;
     // prepare location
-    _ClassMemberLocation targetLocation =
-        _prepareNewGetterLocation(targetClassNode);
+    ClassMemberLocation targetLocation =
+        utils.prepareNewGetterLocation(targetClassNode);
     // build method source
     String targetFile = targetClassElement.source.fullName;
     SourceBuilder sb = new SourceBuilder(targetFile, targetLocation.offset);
@@ -1186,9 +1188,11 @@ class FixProcessor {
   }
 
   void _addFix_createImportUri() {
+    // TODO(brianwilkerson) Generalize this to allow other valid string literals.
+    // TODO(brianwilkerson) Support the case where the node's parent is a Configuration.
     if (node is SimpleStringLiteral && node.parent is ImportDirective) {
       ImportDirective importDirective = node.parent;
-      Source source = importDirective.source;
+      Source source = importDirective.uriSource;
       if (source != null) {
         String file = source.fullName;
         if (isAbsolute(file) && AnalysisEngine.isDartFileName(file)) {
@@ -1278,7 +1282,7 @@ class FixProcessor {
     // EOL management
     bool isFirst = true;
     void addEolIfNotFirst() {
-      if (!isFirst || _isClassWithEmptyBody(targetClass)) {
+      if (!isFirst || utils.isClassWithEmptyBody(targetClass)) {
         sb.append(eol);
       }
       isFirst = false;
@@ -1406,9 +1410,10 @@ class FixProcessor {
   }
 
   void _addFix_createPartUri() {
+    // TODO(brianwilkerson) Generalize this to allow other valid string literals.
     if (node is SimpleStringLiteral && node.parent is PartDirective) {
       PartDirective partDirective = node.parent;
-      Source source = partDirective.source;
+      Source source = partDirective.uriSource;
       if (source != null) {
         String libName = unitLibraryElement.name;
         SourceEdit edit = new SourceEdit(0, 0, 'part of $libName;$eol$eol');
@@ -1480,7 +1485,7 @@ class FixProcessor {
         // don't add this library again
         alreadyImportedWithPrefix.add(libraryElement.source);
         // update library
-        String newShowCode = 'show ${StringUtils.join(showNames, ", ")}';
+        String newShowCode = 'show ${showNames.join(', ')}';
         _addReplaceEdit(
             rf.rangeOffsetEnd(showCombinator), newShowCode, unitLibraryElement);
         _addFix(DartFixKind.IMPORT_LIBRARY_SHOW, [libraryName]);
@@ -1549,8 +1554,22 @@ class FixProcessor {
         if (element.kind != kind) {
           continue;
         }
-        _addFix_importLibrary(
-            DartFixKind.IMPORT_LIBRARY_PROJECT, libraryElement);
+        // Compute the fix kind.
+        FixKind fixKind;
+        if (resourceProvider.pathContext
+            .split(librarySource.fullName)
+            .contains('src')) {
+          // Bad: non-API.
+          fixKind = DartFixKind.IMPORT_LIBRARY_PROJECT3;
+        } else if (element.library != libraryElement) {
+          // Ugly: exports.
+          fixKind = DartFixKind.IMPORT_LIBRARY_PROJECT2;
+        } else {
+          // Good: direct declaration.
+          fixKind = DartFixKind.IMPORT_LIBRARY_PROJECT1;
+        }
+        // Add the fix.
+        _addFix_importLibrary(fixKind, libraryElement);
       }
     }
   }
@@ -1803,31 +1822,44 @@ class FixProcessor {
   }
 
   void _addFix_undefinedClass_useSimilar() {
+    AstNode node = this.node;
+    // Prepare the optional import prefix name.
+    String prefixName = null;
+    if (node is SimpleIdentifier && node.staticElement is PrefixElement) {
+      AstNode parent = node.parent;
+      if (parent is PrefixedIdentifier &&
+          parent.prefix == node &&
+          parent.parent is TypeName) {
+        prefixName = (node as SimpleIdentifier).name;
+        node = parent.identifier;
+      }
+    }
+    // Process if looks like a type.
     if (_mayBeTypeIdentifier(node)) {
+      // Prepare for selecting the closest element.
       String name = (node as SimpleIdentifier).name;
       _ClosestElementFinder finder = new _ClosestElementFinder(
           name,
           (Element element) => element is ClassElement,
           MAX_LEVENSHTEIN_DISTANCE);
-      // find closest element
-      {
-        // elements of this library
+      // Check elements of this library.
+      if (prefixName == null) {
         for (CompilationUnitElement unit in unitLibraryElement.units) {
           finder._updateList(unit.types);
         }
-        // elements from imports
-        for (ImportElement importElement in unitLibraryElement.imports) {
-          if (importElement.prefix == null) {
-            Map<String, Element> namespace = getImportNamespace(importElement);
-            finder._updateList(namespace.values);
-          }
+      }
+      // Check elements from imports.
+      for (ImportElement importElement in unitLibraryElement.imports) {
+        if (importElement.prefix?.name == prefixName) {
+          Map<String, Element> namespace = getImportNamespace(importElement);
+          finder._updateList(namespace.values);
         }
       }
-      // if we have close enough element, suggest to use it
+      // If we have a close enough element, suggest to use it.
       if (finder._element != null) {
         String closestName = finder._element.name;
         _addReplaceEdit(rf.rangeNode(node), closestName);
-        // add proposal
+        // Add proposal.
         if (closestName != null) {
           _addFix(DartFixKind.CHANGE_TO, [closestName]);
         }
@@ -1937,24 +1969,39 @@ class FixProcessor {
   }
 
   void _addFix_undefinedFunction_useSimilar() {
+    AstNode node = this.node;
     if (node is SimpleIdentifier) {
-      String name = (node as SimpleIdentifier).name;
+      // Prepare the optional import prefix name.
+      String prefixName = null;
+      {
+        AstNode invocation = node.parent;
+        if (invocation is MethodInvocation && invocation.methodName == node) {
+          Expression target = invocation.target;
+          if (target is SimpleIdentifier &&
+              target.staticElement is PrefixElement) {
+            prefixName = target.name;
+          }
+        }
+      }
+      // Prepare for selecting the closest element.
       _ClosestElementFinder finder = new _ClosestElementFinder(
-          name,
+          node.name,
           (Element element) => element is FunctionElement,
           MAX_LEVENSHTEIN_DISTANCE);
-      // this library
-      for (CompilationUnitElement unit in unitLibraryElement.units) {
-        finder._updateList(unit.functions);
+      // Check to this library units.
+      if (prefixName == null) {
+        for (CompilationUnitElement unit in unitLibraryElement.units) {
+          finder._updateList(unit.functions);
+        }
       }
-      // imports
+      // Check unprefixed imports.
       for (ImportElement importElement in unitLibraryElement.imports) {
-        if (importElement.prefix == null) {
+        if (importElement.prefix?.name == prefixName) {
           Map<String, Element> namespace = getImportNamespace(importElement);
           finder._updateList(namespace.values);
         }
       }
-      // if we have close enough element, suggest to use it
+      // If we have a close enough element, suggest to use it.
       if (finder._element != null) {
         String closestName = finder._element.name;
         _addReplaceEdit(rf.rangeNode(node), closestName);
@@ -2733,64 +2780,6 @@ class FixProcessor {
     return node is SimpleIdentifier && node.name == 'await';
   }
 
-  /**
-   * Return `true` if the given [classDeclaration] has open '{' and close '}'
-   * at the same line, e.g. `class X {}`.
-   */
-  bool _isClassWithEmptyBody(ClassDeclaration classDeclaration) {
-    return utils.getLineThis(classDeclaration.leftBracket.offset) ==
-        utils.getLineThis(classDeclaration.rightBracket.offset);
-  }
-
-  _ClassMemberLocation _prepareNewClassMemberLocation(
-      ClassDeclaration classDeclaration,
-      bool shouldSkip(ClassMember existingMember)) {
-    String indent = utils.getIndent(1);
-    // Find the last target member.
-    ClassMember targetMember = null;
-    List<ClassMember> members = classDeclaration.members;
-    for (ClassMember member in members) {
-      if (shouldSkip(member)) {
-        targetMember = member;
-      } else {
-        break;
-      }
-    }
-    // After the last target member.
-    if (targetMember != null) {
-      return new _ClassMemberLocation(eol + eol + indent, targetMember.end, '');
-    }
-    // At the beginning of the class.
-    String suffix = members.isNotEmpty ||
-        _isClassWithEmptyBody(classDeclaration) ? eol : '';
-    return new _ClassMemberLocation(
-        eol + indent, classDeclaration.leftBracket.end, suffix);
-  }
-
-  _ClassMemberLocation _prepareNewConstructorLocation(
-      ClassDeclaration classDeclaration) {
-    return _prepareNewClassMemberLocation(
-        classDeclaration,
-        (member) =>
-            member is FieldDeclaration || member is ConstructorDeclaration);
-  }
-
-  _ClassMemberLocation _prepareNewFieldLocation(
-      ClassDeclaration classDeclaration) {
-    return _prepareNewClassMemberLocation(
-        classDeclaration, (member) => member is FieldDeclaration);
-  }
-
-  _ClassMemberLocation _prepareNewGetterLocation(
-      ClassDeclaration classDeclaration) {
-    return _prepareNewClassMemberLocation(
-        classDeclaration,
-        (member) =>
-            member is FieldDeclaration ||
-            member is ConstructorDeclaration ||
-            member is MethodDeclaration && member.isGetter);
-  }
-
   /**
    * Removes any [ParenthesizedExpression] enclosing [expr].
    *
@@ -2909,17 +2898,6 @@ class LintNames {
   static const String annotate_overrides = 'annotate_overrides';
 }
 
-/**
- * Describes the location for a newly created [ClassMember].
- */
-class _ClassMemberLocation {
-  final String prefix;
-  final int offset;
-  final String suffix;
-
-  _ClassMemberLocation(this.prefix, this.offset, this.suffix);
-}
-
 /**
  * Helper for finding [Element] with name closest to the given.
  */
