@@ -6,6 +6,7 @@ import 'package:unittest/unittest.dart';
 
 import '../../utils.dart';
 import 'index_test.dart' as index_test;
+import 'index_unit_test.dart' as index_unit_test;
 
 /**
  * Utility for manually running all tests.
@@ -14,5 +15,6 @@ main() {
   initializeTestEnvironment();
   group('index', () {
     index_test.main();
+    index_unit_test.main();
   });
 }
