@@ -54,11 +54,20 @@ abstract class File implements Resource {
   File renameSync(String newPath);
 
   /**
-   * Synchronously write a list of bytes to the file.
+   * Synchronously write the given [bytes] to the file. The new content will
+   * replace any existing content.
    *
    * Throws a [FileSystemException] if the operation fails.
    */
   void writeAsBytesSync(List<int> bytes);
+
+  /**
+   * Synchronously write the given [content] to the file. The new content will
+   * replace any existing content.
+   *
+   * Throws a [FileSystemException] if the operation fails.
+   */
+  void writeAsStringSync(String content);
 }
 
 /**
@@ -167,6 +176,12 @@ abstract class Resource {
    */
   bool isOrContains(String path);
 
+  /**
+   * Return a resource that refers to the same resource as this resource, but
+   * whose path does not contain any symbolic links.
+   */
+  Resource resolveSymbolicLinksSync();
+
   /**
    * Return a Uri representing this resource.
    */
