@@ -5,11 +5,11 @@
 library analyzer.src.context.context_builder;
 
 import 'dart:collection';
-import 'dart:core' hide Resource;
-import 'dart:io' as io;
+import 'dart:core';
 
 import 'package:analyzer/context/declared_variables.dart';
 import 'package:analyzer/file_system/file_system.dart';
+import 'package:analyzer/plugin/options.dart';
 import 'package:analyzer/plugin/resolver_provider.dart';
 import 'package:analyzer/source/analysis_options_provider.dart';
 import 'package:analyzer/source/package_map_resolver.dart';
@@ -18,10 +18,10 @@ import 'package:analyzer/src/generated/engine.dart';
 import 'package:analyzer/src/generated/sdk.dart';
 import 'package:analyzer/src/generated/source.dart';
 import 'package:analyzer/src/task/options.dart';
-import 'package:package_config/discovery.dart';
 import 'package:package_config/packages.dart';
 import 'package:package_config/packages_file.dart';
 import 'package:package_config/src/packages_impl.dart';
+import 'package:path/src/context.dart';
 import 'package:yaml/yaml.dart';
 
 /**
@@ -60,7 +60,7 @@ class ContextBuilder {
   final DartSdkManager sdkManager;
 
   /**
-   * The cache containing the contents of overlayed files.
+   * The cache containing the contents of overlaid files.
    */
   final ContentCache contentCache;
 
@@ -123,28 +123,26 @@ class ContextBuilder {
    * *Note:* This method is not yet fully implemented and should not be used.
    */
   AnalysisContext buildContext(String path) {
-    // TODO(brianwilkerson) Split getAnalysisOptions so we can capture the
-    // option map and use it to run the options processors.
-    AnalysisOptions options = getAnalysisOptions(path);
     InternalAnalysisContext context =
         AnalysisEngine.instance.createAnalysisContext();
+    AnalysisOptions options = getAnalysisOptions(context, path);
     context.contentCache = contentCache;
     context.sourceFactory = createSourceFactory(path, options);
     context.analysisOptions = options;
+    context.name = path;
     //_processAnalysisOptions(context, optionMap);
     declareVariables(context);
     return context;
   }
 
   Map<String, List<Folder>> convertPackagesToMap(Packages packages) {
-    if (packages == null || packages == Packages.noPackages) {
-      return null;
-    }
     Map<String, List<Folder>> folderMap = new HashMap<String, List<Folder>>();
-    packages.asMap().forEach((String packagePath, Uri uri) {
-      String path = resourceProvider.pathContext.fromUri(uri);
-      folderMap[packagePath] = [resourceProvider.getFolder(path)];
-    });
+    if (packages != null && packages != Packages.noPackages) {
+      packages.asMap().forEach((String packageName, Uri uri) {
+        String path = resourceProvider.pathContext.fromUri(uri);
+        folderMap[packageName] = [resourceProvider.getFolder(path)];
+      });
+    }
     return folderMap;
   }
 
@@ -183,30 +181,30 @@ class ContextBuilder {
 
   Packages createPackageMap(String rootDirectoryPath) {
     if (defaultPackageFilePath != null) {
-      // TODO(brianwilkerson) Figure out why we're going through Uri rather than
-      // just creating the file from the path.
-      Uri fileUri = new Uri.file(defaultPackageFilePath);
-      io.File configFile = new io.File.fromUri(fileUri).absolute;
+      File configFile = resourceProvider.getFile(defaultPackageFilePath);
       List<int> bytes = configFile.readAsBytesSync();
-      Map<String, Uri> map = parse(bytes, configFile.uri);
+      Map<String, Uri> map = parse(bytes, configFile.toUri());
+      resolveSymbolicLinks(map);
       return new MapPackages(map);
     } else if (defaultPackagesDirectoryPath != null) {
-      return getPackagesDirectory(
-          new Uri.directory(defaultPackagesDirectoryPath));
+      Folder folder = resourceProvider.getFolder(defaultPackagesDirectoryPath);
+      return getPackagesFromFolder(folder);
     }
-    return findPackagesFromFile(new Uri.directory(rootDirectoryPath));
+    return findPackagesFromFile(rootDirectoryPath);
   }
 
   SourceFactory createSourceFactory(
       String rootDirectoryPath, AnalysisOptions options) {
     Folder _folder = null;
     Folder folder() {
-      return _folder ??= resourceProvider.getResource('.');
+      return _folder ??= resourceProvider.getFolder(rootDirectoryPath);
     }
 
-    UriResolver fileResolver = fileResolverProvider == null
-        ? new ResourceUriResolver(resourceProvider)
-        : fileResolverProvider(folder());
+    UriResolver fileResolver;
+    if (fileResolverProvider != null) {
+      fileResolver = fileResolverProvider(folder());
+    }
+    fileResolver ??= new ResourceUriResolver(resourceProvider);
     if (packageResolverProvider != null) {
       UriResolver packageResolver = packageResolverProvider(folder());
       if (packageResolver != null) {
@@ -218,18 +216,17 @@ class ContextBuilder {
           packageResolver,
           fileResolver
         ];
-        return new SourceFactory(resolvers);
+        return new SourceFactory(resolvers, null, resourceProvider);
       }
     }
-    Map<String, List<Folder>> packageMap =
-        convertPackagesToMap(createPackageMap(rootDirectoryPath));
-    List<UriResolver> resolvers = <UriResolver>[];
-    resolvers.add(new DartUriResolver(findSdk(packageMap, options)));
-    if (packageMap != null) {
-      resolvers.add(new PackageMapUriResolver(resourceProvider, packageMap));
-    }
-    resolvers.add(fileResolver);
-    return new SourceFactory(resolvers);
+    Packages packages = createPackageMap(rootDirectoryPath);
+    Map<String, List<Folder>> packageMap = convertPackagesToMap(packages);
+    List<UriResolver> resolvers = <UriResolver>[
+      new DartUriResolver(findSdk(packageMap, options)),
+      new PackageMapUriResolver(resourceProvider, packageMap),
+      fileResolver
+    ];
+    return new SourceFactory(resolvers, packages, resourceProvider);
   }
 
   /**
@@ -245,6 +242,30 @@ class ContextBuilder {
     }
   }
 
+  /**
+   * Finds a package resolution strategy for the directory at the given absolute
+   * [path].
+   *
+   * This function first tries to locate a `.packages` file in the directory. If
+   * that is not found, it instead checks for the presence of a `packages/`
+   * directory in the same place. If that also fails, it starts checking parent
+   * directories for a `.packages` file, and stops if it finds it. Otherwise it
+   * gives up and returns [Packages.noPackages].
+   */
+  Packages findPackagesFromFile(String path) {
+    Resource location = _findPackagesLocation(path);
+    if (location is File) {
+      List<int> fileBytes = location.readAsBytesSync();
+      Map<String, Uri> map =
+          parse(fileBytes, resourceProvider.pathContext.toUri(location.path));
+      resolveSymbolicLinks(map);
+      return new MapPackages(map);
+    } else if (location is Folder) {
+      return getPackagesFromFolder(location);
+    }
+    return Packages.noPackages;
+  }
+
   /**
    * Return the SDK that should be used to analyze code. Use the given
    * [packageMap] and [options] to locate the SDK.
@@ -312,16 +333,24 @@ class ContextBuilder {
   }
 
   /**
-   * Return the analysis options that should be used when analyzing code in the
-   * directory with the given [path].
+   * Return the analysis options that should be used when the given [context] is
+   * used to analyze code in the directory with the given [path].
    */
-  AnalysisOptions getAnalysisOptions(String path) {
+  AnalysisOptions getAnalysisOptions(AnalysisContext context, String path) {
     AnalysisOptionsImpl options = createDefaultOptions();
     File optionsFile = getOptionsFile(path);
     if (optionsFile != null) {
-      Map<String, YamlNode> fileOptions =
-          new AnalysisOptionsProvider().getOptionsFromFile(optionsFile);
-      applyToAnalysisOptions(options, fileOptions);
+      List<OptionsProcessor> optionsProcessors =
+          AnalysisEngine.instance.optionsPlugin.optionsProcessors;
+      try {
+        Map<String, YamlNode> optionMap =
+            new AnalysisOptionsProvider().getOptionsFromFile(optionsFile);
+        optionsProcessors.forEach(
+            (OptionsProcessor p) => p.optionsProcessed(context, optionMap));
+        applyToAnalysisOptions(options, optionMap);
+      } on Exception catch (exception) {
+        optionsProcessors.forEach((OptionsProcessor p) => p.onError(exception));
+      }
     }
     return options;
   }
@@ -349,6 +378,103 @@ class ContextBuilder {
     }
     return null;
   }
+
+  /**
+   * Create a [Packages] object for a 'package' directory ([folder]).
+   *
+   * Package names are resolved as relative to sub-directories of the package
+   * directory.
+   */
+  Packages getPackagesFromFolder(Folder folder) {
+    Context pathContext = resourceProvider.pathContext;
+    Map<String, Uri> map = new HashMap<String, Uri>();
+    for (Resource child in folder.getChildren()) {
+      if (child is Folder) {
+        // Inline resolveSymbolicLinks for performance reasons.
+        String packageName = pathContext.basename(child.path);
+        String folderPath = resolveSymbolicLink(child);
+        String uriPath = pathContext.join(folderPath, '.');
+        map[packageName] = pathContext.toUri(uriPath);
+      }
+    }
+    return new MapPackages(map);
+  }
+
+  /**
+   * Resolve any symbolic links encoded in the path to the given [folder].
+   */
+  String resolveSymbolicLink(Folder folder) {
+    try {
+      return folder.resolveSymbolicLinksSync().path;
+    } on FileSystemException {
+      return folder.path;
+    }
+  }
+
+  /**
+   * Resolve any symbolic links encoded in the URI's in the given [map] by
+   * replacing the values in the map.
+   */
+  void resolveSymbolicLinks(Map<String, Uri> map) {
+    Context pathContext = resourceProvider.pathContext;
+    for (String packageName in map.keys) {
+      Folder folder =
+          resourceProvider.getFolder(pathContext.fromUri(map[packageName]));
+      String folderPath = resolveSymbolicLink(folder);
+      // Add a '.' so that the URI is suitable for resolving relative URI's
+      // against it.
+      String uriPath = pathContext.join(folderPath, '.');
+      map[packageName] = pathContext.toUri(uriPath);
+    }
+  }
+
+  /**
+   * Find the location of the package resolution file/directory for the
+   * directory at the given absolute [path].
+   *
+   * Checks for a `.packages` file in the [path]. If not found,
+   * checks for a `packages` directory in the same directory. If still not
+   * found, starts checking parent directories for `.packages` until reaching
+   * the root directory.
+   *
+   * Return a [File] object representing a `.packages` file if one is found, a
+   * [Folder] object for the `packages/` directory if that is found, or `null`
+   * if neither is found.
+   */
+  Resource _findPackagesLocation(String path) {
+    Folder folder = resourceProvider.getFolder(path);
+    if (!folder.exists) {
+      throw new ArgumentError.value(path, "path", "Directory does not exist.");
+    }
+    File checkForConfigFile(Folder folder) {
+      File file = folder.getChildAssumingFile('.packages');
+      if (file.exists) {
+        return file;
+      }
+      return null;
+    }
+
+    // Check for $cwd/.packages
+    File packagesCfgFile = checkForConfigFile(folder);
+    if (packagesCfgFile != null) {
+      return packagesCfgFile;
+    }
+    // Check for $cwd/packages/
+    Folder packagesDir = folder.getChildAssumingFolder("packages");
+    if (packagesDir.exists) {
+      return packagesDir;
+    }
+    // Check for cwd(/..)+/.packages
+    Folder parentDir = folder.parent;
+    while (parentDir != null) {
+      packagesCfgFile = checkForConfigFile(parentDir);
+      if (packagesCfgFile != null) {
+        return packagesCfgFile;
+      }
+      parentDir = parentDir.parent;
+    }
+    return null;
+  }
 }
 
 /**
@@ -378,7 +504,7 @@ class EmbedderYamlLocator {
   }
 
   /**
-   * Programatically add an `_embedder.yaml` mapping.
+   * Programmatically add an `_embedder.yaml` mapping.
    */
   void addEmbedderYaml(Folder libDir, String embedderYaml) {
     _processEmbedderYaml(libDir, embedderYaml);
