@@ -12,7 +12,6 @@ import 'package:analyzer/src/dart/element/element.dart';
 import 'package:analyzer/src/dart/element/type.dart';
 import 'package:analyzer/src/generated/engine.dart'
     show AnalysisContext, AnalysisEngine;
-import 'package:analyzer/src/generated/java_core.dart';
 import 'package:analyzer/src/generated/java_engine.dart';
 import 'package:analyzer/src/generated/source.dart';
 import 'package:analyzer/src/generated/utilities_dart.dart';
@@ -115,9 +114,6 @@ class ConstructorMember extends ExecutableMember implements ConstructorElement {
     List<DartType> parameterTypes = definingType.element.type.typeArguments;
     FunctionType substitutedType =
         baseType.substitute2(argumentTypes, parameterTypes);
-    if (baseType == substitutedType) {
-      return constructor;
-    }
     return new ConstructorMember(constructor, definingType, substitutedType);
   }
 }
@@ -149,7 +145,7 @@ abstract class ExecutableMember extends Member implements ExecutableElement {
     // Elements within this element should have type parameters substituted,
     // just like this element.
     //
-    throw new UnsupportedOperationException();
+    throw new UnsupportedError('functions');
 //    return getBaseElement().getFunctions();
   }
 
@@ -186,7 +182,7 @@ abstract class ExecutableMember extends Member implements ExecutableElement {
     // Elements within this element should have type parameters substituted,
     // just like this element.
     //
-    throw new UnsupportedOperationException();
+    throw new UnsupportedError('localVariables');
 //    return getBaseElement().getLocalVariables();
   }
 
@@ -233,6 +229,9 @@ class FieldFormalParameterMember extends ParameterMember
       [DartType type])
       : super(baseElement, definingType, type);
 
+  @override
+  bool get isCovariant => baseElement.isCovariant;
+
   @override
   FieldElement get field {
     FieldElement field = (baseElement as FieldFormalParameterElement).field;
@@ -273,6 +272,9 @@ class FieldMember extends VariableMember implements FieldElement {
   @override
   bool get isEnumConstant => baseElement.isEnumConstant;
 
+  @override
+  bool get isVirtual => baseElement.isVirtual;
+
   @override
   DartType get propagatedType => substituteFor(baseElement.propagatedType);
 
@@ -403,9 +405,6 @@ class FunctionMember extends ExecutableMember implements FunctionElement {
         TypeParameterTypeImpl.getTypes(definingType.typeParameters);
     FunctionType substitutedType =
         baseType.substitute2(argumentTypes, parameterTypes);
-    if (baseType == substitutedType) {
-      return method;
-    }
     return new MethodMember(method, definingType, substitutedType);
   }
 }
@@ -447,10 +446,6 @@ abstract class Member implements Element {
   @override
   String get displayName => _baseElement.displayName;
 
-  @deprecated
-  @override
-  SourceRange get docRange => _baseElement.docRange;
-
   @override
   String get documentationComment => _baseElement.documentationComment;
 
@@ -644,9 +639,6 @@ class MethodMember extends ExecutableMember implements MethodElement {
     List<DartType> parameterTypes = definingType.element.type.typeArguments;
     FunctionType substitutedType =
         baseType.substitute2(argumentTypes, parameterTypes);
-    if (baseType == substitutedType) {
-      return method;
-    }
     return new MethodMember(method, definingType, substitutedType);
   }
 }
@@ -679,6 +671,9 @@ class ParameterMember extends VariableMember
   @override
   int get hashCode => baseElement.hashCode;
 
+  @override
+  bool get isCovariant => baseElement.isCovariant;
+
   @override
   bool get isInitializingFormal => baseElement.isInitializingFormal;
 
@@ -772,9 +767,6 @@ class ParameterMember extends VariableMember
           TypeParameterTypeImpl.getTypes(definingType.typeParameters);
       DartType substitutedType =
           baseType.substitute2(argumentTypes, parameterTypes);
-      if (baseType == substitutedType) {
-        return parameter;
-      }
       return new ParameterMember(parameter, definingType, substitutedType);
     }
   }
@@ -937,9 +929,13 @@ class TypeParameterMember extends Member implements TypeParameterElement {
   @override
   final DartType bound;
 
+  DartType _type;
+
   TypeParameterMember(
       TypeParameterElement baseElement, DartType definingType, this.bound)
-      : super(baseElement, definingType);
+      : super(baseElement, definingType) {
+    _type = new TypeParameterTypeImpl(this);
+  }
 
   @override
   TypeParameterElement get baseElement =>
@@ -949,7 +945,16 @@ class TypeParameterMember extends Member implements TypeParameterElement {
   Element get enclosingElement => baseElement.enclosingElement;
 
   @override
-  TypeParameterType get type => baseElement.type;
+  int get hashCode => baseElement.hashCode;
+
+  @override
+  TypeParameterType get type => _type;
+
+  @override
+  bool operator ==(obj) =>
+      // TODO(jmesserly): this equality should consider the bound, see:
+      // https://github.com/dart-lang/sdk/issues/27210
+      obj is TypeParameterMember && obj.baseElement == baseElement;
 
   @override
   accept(ElementVisitor visitor) => visitor.visitTypeParameterElement(this);
@@ -973,9 +978,6 @@ class TypeParameterMember extends Member implements TypeParameterElement {
         TypeParameterTypeImpl.getTypes(definingType.typeParameters);
     DartType substitutedBound =
         bound.substitute2(argumentTypes, parameterTypes);
-    if (bound == substitutedBound) {
-      return parameter;
-    }
     return new TypeParameterMember(parameter, definingType, substitutedBound);
   }
 }
@@ -1020,7 +1022,7 @@ abstract class VariableMember extends Member implements VariableElement {
     // Elements within this element should have type parameters substituted,
     // just like this element.
     //
-    throw new UnsupportedOperationException();
+    throw new UnsupportedError('initializer');
     //    return getBaseElement().getInitializer();
   }
 
