@@ -7,14 +7,14 @@ import 'package:analyzer/src/dart/element/element.dart';
 import 'package:analyzer/src/summary/format.dart';
 import 'package:analyzer/src/summary/idl.dart';
 import 'package:analyzer/src/summary/link.dart';
+import 'package:test_reflective_loader/test_reflective_loader.dart';
 import 'package:unittest/unittest.dart';
 
-import '../../reflective_tests.dart';
 import 'summarize_ast_test.dart';
 
 main() {
   groupSep = ' | ';
-  runReflectiveTests(LinkerUnitTest);
+  defineReflectiveTests(LinkerUnitTest);
 }
 
 @reflectiveTest
@@ -33,8 +33,10 @@ class LinkerUnitTest extends SummaryLinkerTest {
 
   void createLinker(String text, {String path: '/test.dart'}) {
     linkerInputs = createLinkerInputs(text, path: path);
-    Map<String, LinkedLibraryBuilder> linkedLibraries =
-        setupForLink(linkerInputs.linkedLibraries, linkerInputs.getUnit);
+    Map<String, LinkedLibraryBuilder> linkedLibraries = setupForLink(
+        linkerInputs.linkedLibraries,
+        linkerInputs.getUnit,
+        linkerInputs.getDeclaredVariable);
     linker = new Linker(linkedLibraries, linkerInputs.getDependency,
         linkerInputs.getUnit, true);
   }
