@@ -8,7 +8,6 @@ import 'package:unittest/unittest.dart';
 
 import 'cancelable_future_test.dart' as cancelable_future_test;
 import 'context/test_all.dart' as context;
-import 'enum_test.dart' as enum_test;
 import 'file_system/test_all.dart' as file_system;
 import 'generated/test_all.dart' as generated;
 import 'instrumentation/test_all.dart' as instrumentation;
@@ -23,7 +22,6 @@ main() {
   group('analysis engine', () {
     cancelable_future_test.main();
     context.main();
-    enum_test.main();
     file_system.main();
     generated.main();
     instrumentation.main();
